/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: VLTItable.h,v 1.9 2010/01/24 10:57:38 jaffe Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jdejong 2002-05-08  imported
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

/* $Id: VLTItable.h,v 1.9 2010/01/24 10:57:38 jaffe Exp $ */

#ifndef _VLTITABLE_H
#define _VLTITABLE_H
/*----------------------------------------------------------------------- */
/* FITS binary tables routines for the VLTI interferometer                */
/*----------------------------------------------------------------------- */
/* Table definition class for FITS binary tables.                         */
/* This package creates and maintains table definition structures used to */
/* create FITS binary tables.                                             */
/* A new table type XXX requires: (mostly in table_*_def.c)               */
/*  1) a unique definition of XXX_TABLE in VLTItable.h                    */  
/*  2) prototype and definition of make_XXX_table() which includes XXX.def*/
/*     in VLTIprivate.h and table_def_priv.c                              */ 
/*  3) an XXX.def file defining the components of the table in ../include */
/*  4) add case for table type in make_table_def                          */
/*     in table_def_pub.c                                                 */
/*----------------------------------------------------------------------- */
#include "fitsio.h"
#include <stdlib.h>
#include <string.h>

/* defines for  known table types */
/* add new values for a new table type */
#define  XXX_TABLE    0 /* as an example */
#define  SOURCE_TABLE 1
#define  ARRAY_GEOMETRY_TABLE 2
#define  ADAPTIVE_OPTICS_TABLE 3
#define  FLAG_TABLE 4
#define  DELAY_TABLE 5
#define  IMAGING_DATA_TABLE 6
#define  IMAGING_DETECTOR_TABLE 7
#define  PREVISIBILITY_TABLE 8
#define  VISIBILITY_TABLE 9
#define  TIP_TILT_TABLE 10
#define  CALIBRATION_TABLE 11
#define  OPTICAL_TRAIN_TABLE 12
#define  IMAGING_FDATA_TABLE 13
#define  WAVELENGTH_TABLE 14
#define  OI_VIS_TABLE 15
#define  DISPERSION_TABLE 16
#define  OI_TARGET_TABLE 17

typedef struct      /* structure used to store table column information */
{
/*  int   tcol;        column number */
/*  char* ttype;       column name = FITS TTYPEn keyword; */
/*  int   tdatatype;   datatype code of each column (see fitsio.h)*/
/*  long  tcount;      number of elements */
  union {           /* allow references by correct type */
    void*   tvoid;
    char*   tchar;
    short*  tshort;
    int*    tint;
    long*   tlong;
    float*  tfloat;
    double* tdouble;
  } tdata;            /* pointer to data */
} table_col;

typedef struct        /* structure used to store table row information */
{
  long trow;        /* row number */
  int  ncol;        /* Number of columns in row */
  table_col **data; /* array of column data */
} table_row;

/* table definition structure */
typedef struct
{
  char *extname;      /* table type */
  int  nkey;          /* Number of keywords */
  char** keywords;    /* array of keyword names */
  int*  keytype;      /* type codes for keywords, as defined in fitsio.h */
  char** keycom;      /* Comments for keywords */
  void** keydata;     /* Data array for keywords as void */
  int  ncol;          /* number of columns */
  char** coltype;     /* column names = FITS TTYPE; */
  long* colcount;     /* Number of elements in each column. */
  int*  coldatatype;  /* datatype code of each column (see fitsio.h)*/
  char** colunits;    /* column units = FITS TUNIT* keywords; */
  char** coldim;      /* dimensionality strings, null means scalar or vector */
} table_def;

/* public functions */
/* generic table functions */
/* there are more complete descriptions of the functions in the source file */

table_col* make_table_col(void* tdata);
/*----------------------------------------------------------------------- */
/*   Make a table_col object                                              */
/*   Inputs:                                                              */
/*      tdata      Column data array                                      */
/*   Returns:                                                             */
/*      New table_col structure; NULL -> failed                           */
/*----------------------------------------------------------------------- */

table_row* make_table_row(long  trow, int ncol);
/*----------------------------------------------------------------------- */
/*   Make a table_row object                                              */
/*   Inputs:                                                              */
/*      trow       Row number number                                      */
/*      ncol       Number of columns                                      */
/*   Returns:                                                             */
/*      New table_row structure; NULL -> failed                           */
/*----------------------------------------------------------------------- */

/* destroy a table_col object */
void kill_table_col(table_col* col);
/*----------------------------------------------------------------------- */
/*   Destroy a table_col object                                           */
/*   This is normally done as part of table_row destruction.              */
/*   Inputs:                                                              */
/*      col  Object to destroy                                            */
/*----------------------------------------------------------------------- */

void kill_table_row(table_row* row);
/*----------------------------------------------------------------------- */
/*   Destroy a table_row object                                           */
/*   Inputs:                                                              */
/*      table_row  Object to destroy                                      */
/*----------------------------------------------------------------------- */

void make_table (fitsfile *fptr, long naxis2, int tfields, 
		      char* ttype[], char* tform[], char* tunit[], 
		      char* extname, 
		      int nkey, int keytype[], char* keyword[], 
		      void* keyvalue[], char* keycomment[], 
		      int *ierr);
/*----------------------------------------------------------------------- */
/*   Function to create and initialize a FITS binary table                */
/*   Inputs:                                                              */
/*      fptr       Pointer to open FITSIO file structure                  */
/*      naxis2     Minimum number of rows, generally use 0                */
/*      tfields    Number of columns in table                             */
/*      ttype      Column labels (tfield values).                         */
/*      tform      Column formats                                         */
/*      tunit      Column units                                           */
/*      extname    Name of table type                                     */
/*      nkey       Number of additional keywords in header                */
/*      keytype    Keyword data types as defined in fitsio.h              */
/*      keyword    Keywords.                                              */
/*      keyvalue   Pointers to values cast to void                        */
/*      keycomment Comment explaining keyword                             */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, returns if initially positive      */
/*----------------------------------------------------------------------- */

void table_print_error_stack ();
/*----------------------------------------------------------------------- */
/*   print cfitsio error stack to stderr                                  */
/*----------------------------------------------------------------------- */

table_def* make_table_def(int table_type, char  colsymb, int symcount);
/*----------------------------------------------------------------------- */
/*   Make a table_def object by type                                      */
/*   Table data storage is allocated                                      */
/*   Inputs:                                                              */
/*      table_type   Table type code as defined above                     */
/*      colsym       A character to be substituted in column names        */
/*                   0 => don't check                                     */
/*      symcount     The maximum integer to replace colsym with           */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure;                  */
/*      NULL indicates failure                                            */
/*----------------------------------------------------------------------- */

void kill_table_def(table_def* tabledef);
/*----------------------------------------------------------------------- */
/*   Destroy a table_def object                                           */
/*   Inputs:                                                              */
/*      tabledef   Pointer to the structure to destroy.                   */
/*----------------------------------------------------------------------- */

void make_table_from_def (fitsfile *fptr, table_def *tabledef,
		      int *ierr);
/*----------------------------------------------------------------------- */
/*   Create and initialize a FITS binary table in an open FITS file       */
/*   Template values from the definitions are used which may be modified  */
/*   as necessary to get array dimensionality correct.                    */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      tabledef  definition structure for table                          */
/*   Outputs:                                                             */
/*      ierr      Error code, 0 = OK, else failed, checked on input       */
/*----------------------------------------------------------------------- */

table_row* make_table_row_from_def (table_def *tabledef);
/*----------------------------------------------------------------------- */
/*   function to create a row structure from a table defintion.           */
/*   Inputs:                                                              */
/*      tabledef  definition structure for table                          */
/*   Returns:                                                             */
/*      Pointer to a fully created table_row structure, NULL=>failed      */
/*----------------------------------------------------------------------- */

void table_write_row (fitsfile *fptr, table_def *tabledef, 
		      table_row *row_data, int *ierr);
/*----------------------------------------------------------------------- */
/*   Function to write a row in the currently open table                  */
/*   Inputs:                                                              */
/*      fptr       Pointer to open FITSIO file structure                  */
/*      tabledef   Pointer to the table defintion structure               */
/*      row_data   Row data structure                                     */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, returns if initially positive      */
/*----------------------------------------------------------------------- */

void table_read_row (fitsfile *fptr, table_def *tabledef, 
		     table_row *row_data, int *ierr);
/*----------------------------------------------------------------------- */
/*   Function to read a row in the currently open table                   */
/*   Inputs:                                                              */
/*      fptr       Pointer to open FITSIO file structure                  */
/*      tabledef   Pointer to the table defintion structure               */
/*      row_data   Row data structure                                     */
/*   Output:                                                              */
/*      row_data   Row data structure filled in                           */
/*      ierr      Error code: 0 => ok, returns if initially positive      */
/*----------------------------------------------------------------------- */

table_def* make_table_def_from_table (fitsfile *fptr);
/*----------------------------------------------------------------------- */
/*   Make a table_def object from an open binary table                    */
/*   Inputs:                                                              */
/*      fptr         cfitsio pointer to open FITS file                    */
/*   Returns:                                                             */
/*      A table_def* for the table definition structure;                  */
/*      NULL indicates failure                                            */
/*----------------------------------------------------------------------- */

void table_get_keyword(table_def *tabledef, int keyword, void *value, 
		       int *ierr);
/*----------------------------------------------------------------------- */
/*   Function to Read a keyword value from a table definition             */
/*   Inputs:                                                              */
/*      tabledef   Pointer to the table definition structure              */
/*      keyword    1-rel keyword number                                   */
/*      value      pointer to memory to recieve value                     */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, returns if initially positive      */
/*                checked on input                                        */
/*----------------------------------------------------------------------- */

void table_put_keyword(table_def *tabledef, int keyword, void *value, 
		       int *ierr);
/*----------------------------------------------------------------------- */
/*   Function to write a keyword value to a table definition              */
/*   Inputs:                                                              */
/*      tabledef   Pointer to the table definition structure              */
/*      keyword    1-rel keyword number                                   */
/*      value      pointer to memory containing value                     */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, returns if initially positive      */
/*----------------------------------------------------------------------- */

char FITS_type_to_code(int type); 
/*----------------------------------------------------------------------- */
/*    Convert integer type to character format code                       */
/*   Input:                                                               */
/*      Data type code as defined in fitsio.h                             */
/*   Returns:                                                             */
/*      The TFORMn character code for the type                           .*/
/*----------------------------------------------------------------------- */

int FITS_code_to_type(char code);
/*----------------------------------------------------------------------- */
/*    Convert character format code to integer type                       */
/*   Input:                                                               */
/*      Data type code as defined in fitsio.h.                            */
/*   Returns:                                                             */
/*      The TFORMn character code for the type.                           */
/*----------------------------------------------------------------------- */

int FITS_type_to_size(int type);
/*----------------------------------------------------------------------- */
/*   Get size of FITS data types                                          */
/*   Input:                                                               */
/*      type  Data type code as defined in fitsio.h.                      */
/*   Returns:                                                             */
/*      The size in bytes.                                                */
/*----------------------------------------------------------------------- */

int table_find_col (table_def *tabledef, char* name);
/*----------------------------------------------------------------------- */
/*   Find the column number of a given label                              */
/*   Inputs:                                                              */
/*      tabledef     Pointer to the table defintion structure             */
/*      name         Desired column label                                 */
/*   Returns:                                                             */
/*      1 - rel column number, -1 => failed                               */
/*----------------------------------------------------------------------- */

int table_find_key (table_def *tabledef, char* name);
/*----------------------------------------------------------------------- */
/*   Find the keyword number of a given label                             */
/*   Inputs:                                                              */
/*      tabledef     Pointer to the table defintion structure             */
/*      name         Desired column label                                 */
/*   Returns:                                                             */
/*      1 - rel column number, -1 => failed                               */
/*----------------------------------------------------------------------- */

/* copy the contents of a string with blank padding */
void padcopy (char *out, char *in, int len);

#endif /* end VLTITABLE_H */
/* */
